from launchpadlib.launchpad import Launchpad
from datetime import datetime, timedelta, date


class DayStats:

    def __init__(self, days_ago, critical_total_closed_today,
            critical_total_opened_today, critical_closed_today=None,
            critical_opened_today=None):
        self.days_ago = days_ago
        self.critical_total_closed_today = critical_total_closed_today
        self.critical_total_opened_today = critical_total_opened_today

        if not critical_closed_today and not critical_opened_today:
            critical_closed_today = critical_total_closed_today
            critical_opened_today = critical_total_opened_today
        self.critical_closed_today = critical_closed_today
        self.critical_opened_today = critical_opened_today

    def update_counts(self, critical_closed_today, critical_opened_today):
        self.critical_closed_today = critical_closed_today
        self.critical_opened_today = critical_opened_today


class Stats:

    def __init__(self):
        self._day_stats = {}

    def add_day(self, days_ago, critical_total_closed_today,
            critical_total_opened_today):
        today_stats = DayStats(days_ago, critical_total_closed_today,
            critical_total_opened_today)
        self._day_stats.setdefault(days_ago, None)
        try:
            next_day_stats = self._day_stats[days_ago-1]

            next_day_stats.critical_closed_today = (
                today_stats.critical_total_closed_today -
                next_day_stats.critical_total_closed_today)
            next_day_stats.critical_opened_today = (
                today_stats.critical_total_opened_today -
                next_day_stats.critical_total_opened_today)
            self._day_stats[days_ago-1] = next_day_stats
        except:
            # There's no other day besides this one.
            pass

        self._day_stats[days_ago] = today_stats



class LaunchpadBugs:

    ALL_STATUSES = [
        u"New",
        u"Incomplete",
        u"Opinion",
        u"Invalid",
        u"Won't Fix",
        u"Expired",
        u"Confirmed",
        u"Triaged",
        u"In Progress",
        u"Fix Committed",
        u"Fix Released",
        u"Unknown",
    ]

    CLOSED_STATUSES = [
        u"Invalid",
        u"Won't Fix",
        u"Expired",
        u"Fix Released",
    ]

    def __init__(self, lp_instance="production", lp=None, days_ago=7,
                 config=None):
        self.days_ago = days_ago
        self.bugs_list = None
        self.lp_instance = lp_instance
        self.config = config
        try:
            if lp is None:
                self.initialize_launchpad()
            else:
                self.lp = lp
            self.launchpad_project = (
                self.lp.project_groups["launchpad-project"])
        except:
            # XXX: Improve this.
            raise

    def _past_date(self, days):
        return datetime.today() - timedelta(days=days)

    def initialize_launchpad(self):
        """Initialize the lp attribute."""
        self.lp = Launchpad.login_with(
            'dashboard', service_root=self.lp_instance,
            credentials_file=self.config.lplib_credentials_file)

    def fetch_open_critical_bugs(self):
        return self.launchpad_project.searchTasks(importance=[u"Critical"])

    def fetch_critical_bugs(self, status=None):
        if status is None:
            status = self.ALL_STATUSES
        else:
            if type(status) != list:
                status = [status]
        return self.launchpad_project.searchTasks(importance=[u"Critical"],
            status=status)

    def fetch_closed_critical_bugs_last_days(self, days=None):
        if days is None:
            days = self.days_ago
        return self.launchpad_project.searchTasks(importance=[u"Critical"],
            modified_since=self._past_date(days), status=self.CLOSED_STATUSES)

    def fetch_new_critical_bugs_last_days(self, days=None):
        if days is None:
            days = self.days_ago
        return self.launchpad_project.searchTasks(importance=[u"Critical"],
            created_since=self._past_date(days))

    def critical_bugs_stats(self, days=None):
        stats = Stats()

        if not days:
            days = self.days_ago

        print days

        for day in range(1, days+1):
            new_critical_since = self.fetch_new_critical_bugs_last_days(day)
            closed_critical_since = self.fetch_closed_critical_bugs_last_days(
                day)
            stats.add_day(day, len(closed_critical_since),
                len(new_critical_since))

        self.stats = stats

    def get_todays_stats(self):
        # Guarantee that time is 00:00:00
        today = datetime.today()
        today_hr = "%d-%02d-%02d" % (today.year, today.month, today.day)
        today = datetime.strptime(today_hr, "%Y-%m-%d")

        total_critical_closed_today = []
        total_critical_today = self.launchpad_project.searchTasks(
            importance=u"Critical", tags='-not-pie-critical')
        total_critical_filed_today = self.launchpad_project.searchTasks(
            importance=u"Critical", created_since=today,
            tags='-not-pie-critical')
        total_critical_modified_today = self.launchpad_project.searchTasks(
            importance=u"Critical", modified_since=today,
            status=self.CLOSED_STATUSES, tags='-not-pie-critical')

        for bugtask in total_critical_modified_today:
            if not bugtask.date_closed:
                continue
            if bugtask.date_closed.strftime("%Y-%m-%d") == today_hr:
                total_critical_closed_today.append(bugtask)

        total_critical_oops = self.launchpad_project.searchTasks(
            importance=u"Critical", tags=[u"oops"])
        total_critical_regressions = self.launchpad_project.searchTasks(
            importance=u"Critical", tags=[u"regression"])
        total_critical_timeouts = self.launchpad_project.searchTasks(
            importance=u"Critical", tags=[u"timeout"])
        total_critical_triaged = self.launchpad_project.searchTasks(
            importance=u"Critical", status=u"Triaged")
        total_critical_fix_committed = self.launchpad_project.searchTasks(
            importance=u"Critical", status=u"Fix Committed")

        self.critical_bugs_filed_today = total_critical_filed_today
        self.critical_bugs_total = total_critical_today
        self.critical_bugs_closed_today = total_critical_closed_today
        self.critical_oopses = total_critical_oops
        self.critical_regressions = total_critical_regressions
        self.critical_timeouts = total_critical_timeouts
        self.critical_triaged = total_critical_triaged
        self.critical_fix_committed = total_critical_fix_committed
        self.critical_today_balance = (len(total_critical_filed_today) -
            len(total_critical_closed_today))

    def get_last_days_stats(self, days=7):
        first_day = datetime.today() - timedelta(days=days)
        first_day_ = datetime.strftime(first_day, "%Y-%m-%d")
        total_critical_closed_last_days = []
        total_critical_filed_last_days = self.launchpad_project.searchTasks(
            importance=u"Critical", created_since=first_day,
            tags='-not-pie-critical')
        total_critical_modified_last_days = self.launchpad_project.searchTasks(
            importance=u"Critical", modified_since=first_day,
            status=self.CLOSED_STATUSES, tags='-not-pie-critical')
        for bugtask in total_critical_modified_last_days:
            if not bugtask.date_closed:
                continue
            if bugtask.date_closed.replace(tzinfo=None) >= first_day:
                total_critical_closed_last_days.append(bugtask)

        self.critical_bugs_filed_last_days = total_critical_filed_last_days
        self.critical_bugs_closed_last_days = total_critical_closed_last_days
        self.critical_last_days_balance = (
            len(total_critical_filed_last_days) - len(
                total_critical_closed_last_days))


