from __future__ import with_statement

__metaclass__ = type

import sys
from datetime import datetime
from os.path import join, dirname, exists
from genshi.template import TemplateLoader
from devops.deploymentreports import DeploymentReport, JenkinsDevelReport
from devops.instancesstatus import Production, Staging, QAStaging
from devops.incidentreports import IncidentReports
from devops.utils import *
from devops.fetchbugs import LaunchpadBugs
from optparse import OptionParser

class Report:

    def __init__(self, config_path):
        self.config_ = Config(config_path)
        self.config_.read_config_file()
        self.devops_index_path = self.config_.devops_index_path
        self.deployment_reports_path = self.config_.deployment_reports_path

    def load_template(self):
        """Retrieve a Genshi HTML template instance for a report."""
        try:
            loader = TemplateLoader(
                join(dirname(__file__), 'templates'))
        except:
            loader = TemplateLoader(join('devops', 'templates'))

        return loader.load('devops-index.html')

    def write_html(self, filename='index.html'):
        """Write an HTML report to a directory."""

        report_timestamp = datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S')

        try:
            dr_stable = DeploymentReport(self.deployment_reports_path,
                'deployment-stable.html')
            dr_stable.get_deployment_qa_data()

            dr_dbstable = DeploymentReport(self.deployment_reports_path,
                'deployment-db-stable.html')
            dr_dbstable.get_deployment_qa_data()

            use_jenkins = True

            try:
                jenkins = JenkinsDevelReport()
                (last_deployed_devel, last_successful_buildno) = \
                    jenkins.last_successful_build
                current_buildno, elapsed_time = jenkins.last_build
            except:
                use_jenkins = False
                last_deployed_devel = last_successful_buildno = None
                current_buildno = elapsed_time = None

            st = Staging()
            st.get_info()
            prod = Production()
            prod.get_info()
            qast = QAStaging()
            qast.get_info()

            incident_reports = IncidentReports()
            incident_reports.get_last_reports_since()

            critical_bugs = LaunchpadBugs(config=self.config_)
            critical_bugs.get_todays_stats()
            critical_bugs.get_last_days_stats()

            context = dict(
                current_summaries_date=yesterday(),
                dr_stable=dr_stable,
                dr_dbstable=dr_dbstable,
                staging_instance=st,
                qastaging_instance=qast,
                prod_instance=prod,
                incident_reports=incident_reports,
                date_generated=report_timestamp,
                critical_bugs_info=critical_bugs,
                use_jenkins=use_jenkins,
                current_buildno=current_buildno,
                elapsed_time=elapsed_time,
                last_deployed_devel=last_deployed_devel,
                last_successful_buildno=last_successful_buildno,
            )

            report_name = filename

            report_path = join(self.devops_index_path, report_name)
            with file(report_path, 'w') as reportfile:
                self.render(reportfile, context)
        except:
            raise

    def render(self, stream, context):
        """Render the report to the given stream using the given context."""
        template = self.load_template()
        template_stream = template.generate(**context)
        stream.write(template_stream.render('html'))


def main(argv=None):
    if argv is None:
        argv = sys.argv

    description = ("This script generates the index page of Launchpad DevOps "
        " Dashboard.")
    usage = "usage: %prog -c CONFIG_FILE"
    parser = OptionParser(description=description, usage=usage)

    # Tagger options
    parser.add_option('-c', '--config', type='string', dest='config_path')

    options, args = parser.parse_args(argv)

    config_path = options.config_path
    if options.config_path:
        if not exists(config_path):
            print "Config file not found: %s" % config_path
            sys.exit(1)
    else:
        config_path = "devops.conf"

    report = Report(config_path)
    report.write_html()


