import tarfile
import tempfile
import os

from bzrlib.branch import Branch

__metaclass__ = type


LPLIB_ROOT="https://api.edge.launchpad.net/beta"
HERE = os.path.abspath(__file__)
TESTDATA_PATH = os.path.normpath(
    os.path.join(
        HERE,
        '../../sample_data/branch_data.tar.bz2'))
OPEN_STATUSES = [u"New", u"Triaged", u"Confirmed", u"Opinion",
    u"Fix Committed"]
ALL_STATUSES = [u"New", u"Incomplete", u"Opinion", u"Invalid", u"Won't Fix",
        u"Expired", u"Confirmed", u"Triaged", u"In Progress",
        u"Fix Committed", u"Fix Released", u"Unknown"]
ALL_IMPORTANCES = [u"Unknown", u"Critical", u"High", u"Medium", u"Low",
    u"Wishlist", u"Undecided"]


class SampleBranch:
    """Convenience class to untar the sample data branch."""

    def __init__(self, testdata_path=None):
        if testdata_path is None:
            testdata_path = TESTDATA_PATH
        tar_file = tarfile.open(TESTDATA_PATH)
        tmp_dir = tempfile.mkdtemp()
        tar_file.extractall(tmp_dir)
        self.path = tmp_dir
        self.branch_name = 'devel'
        self.branch_path = os.path.join(tmp_dir, self.branch_name)
        self.branch = None

    def open_branch(self):
        self.branch = Branch.open(self.branch_path)
        return self.branch


class FakeBugMessage:

    def __init__(self, subject, content):
        self.subject = subject
        self.content = content


class FakeBug:

    def __init__(self, bug_id, title=None, description=None):
        self.id = bug_id
        self.messages = []
        self.title = title
        self.description = description
        # This is how launchpad behaves: adding a message with the title and
        # the description as number 0.
        self.newMessage(self.title, self.description)
        self.tags = []
        self.bug_tasks = []
        self.self_link = "%s/bugs/%s" % (LPLIB_ROOT, bug_id)

    def addBugTask(self, bug_id, project_name="", status=u"New",
            importance=u"Undecided"):
        bug_task = FakeBugTask(bug_id, project_name, status, importance)
        bug_task.bug = self
        bug_task.addTarget(project_name)
        self.bug_tasks.append(bug_task)
        return bug_task

    def newMessage(self, subject, content):
        self.messages.append(FakeBugMessage(subject, content))

    @property
    def message_count(self):
        return len(self.messages)

    def lp_save(self):
        pass


class FakeBugTask:

    def __init__(self, bug_id, project_name, status=u"Fix Committed",
            importance=u"Undecided"):
        self._bug_id = bug_id
        self.bug = None
        self._project_name = project_name
        self.status = status
        self.importance = importance
        self.assignee = None
        self.target = None
        self.milestone = None
        self.tags = []
        self.self_link = "%s/%s/+bug/%s" % (LPLIB_ROOT, project_name, bug_id)

    def addTarget(self, project_name):
        self.target = FakeTarget(project_name)

    def transitionToAssignee(self, assignee):
        self.assignee = assignee

    def transitionToMilestone(self, milestone=None):
        self.milestone = milestone

    def transitionToStatus(self, status=None):
        self.status = status

    def lp_save(self):
        pass


class FakeTarget:

    def __init__(self, project_name):
        self.self_link = "%s/%s" % (LPLIB_ROOT, project_name)


class FakePeople:

    def __init__(self):
        self._people = {}

    def getByEmail(self, email):
        try:
            person = self._people[email]
            return person
        except:
            return None

    def findPerson(self, text=None):
        people_found = []
        for email, person in self._people.iteritems():
            if text in person.display_name:
                people_found.append(person)
        return people_found


class FakePerson:

    def __init__(self, name, display_name, email):
        self.name = name
        self.display_name = display_name
        self.preferred_email = email
        self.self_link = "%s/~%s" % (LPLIB_ROOT, name)


class FakeBranches:

    def __init__(self):
        self._branches = {}

    def getByUrl(self, url):
        try:
            branch = self._branches[url]
            return branch
        except:
            return None


class FakeBranch:

    def __init__(self, branch_nick, owner, project, linked_bugs):
        self.owner = owner
        self.url = "lp:%s/%s/%s" % (owner.name, project, branch_nick)
        self.project = project
        self.linked_bugs = linked_bugs
        self.properties = {}
        self.properties["branch-nick"] = branch_nick


class FakeMilestone:

    def __init__(self, milestone_name, target, active=True):
        self.name = milestone_name
        self.target = target
        self.is_active = active


class FakeProject:

    def __init__(self, project_name, milestone=None):
        self.name = project_name
        self.milestone = milestone
        self.self_link = "%s/%s" % (LPLIB_ROOT, project_name)
        self.bug_tasks = []

    def getMilestone(self, name):
        if self.milestone is not None:
            return FakeMilestone(name, self, True)
        else:
            return None

    def addMilestone(self, name):
        self.milestone = FakeMilestone(name, self, True)

    def searchTasks(self, importance=None, status=None, tags=None):
        # XXX: add tags search support
        search_result = []
        if not importance:
            importance = ALL_IMPORTANCES
        if not status:
            status = OPEN_STATUSES
        for bug in self.bug_tasks:
            if bug.importance in importance and bug.status in status:
                search_result.append(bug)

        return search_result


class FakeProjectGroup:

    def __init__(self, project_group_name):
        self.name = project_group_name
        self.self_link = "%s/%s" % (LPLIB_ROOT, project_group_name)
        self.projects = {}
        # Workaround to be able to use searchTasks within a project group
        self.bug_tasks = []

    def addProject(self, project):
        if type(project) == str:
            self.projects[project] = FakeProject(project)
        else:
            self.projects[project.name] = project

    def searchTasks(self, importance=None, status=None, tags=None):
        # XXX: add tags search support
        search_result = []
        if not importance:
            importance = ALL_IMPORTANCES
        if not status:
            status = OPEN_STATUSES
        for bug in self.bug_tasks:
            if bug.importance in importance and bug.status in status:
                search_result.append(bug)

        return search_result


class FakeLP:

    def __init__(self):
        self.people = FakePeople()
        self.bugs = {}
        self.bug_tasks = []
        self.branches = FakeBranches()
        self.projects = {}
        self.project_groups = {}

    def createProjectGroup(self, project_group_name):
        fake_project_group = FakeProjectGroup(project_group_name)
        self.project_groups[project_group_name] = fake_project_group
        return fake_project_group

    def createBugWithBugTask(self, bug_id, project_name, status, importance,
            title=None, description=None):
        fake_bug = FakeBug(bug_id, title, description)
        bt = fake_bug.addBugTask(bug_id, project_name, status, importance)
        self.bug_tasks.append(bt)
        self.bugs[bug_id] = fake_bug
        return fake_bug

    def createPerson(self, name, display_name, email):
        fake_person = FakePerson(name, display_name, email)
        self.people._people[email] = fake_person
        return fake_person

    def createBranch(self, branch_nick, owner, project, linked_bugs):
        fake_branch = FakeBranch(branch_nick, owner, project, linked_bugs)
        self.branches._branches[fake_branch.url] = fake_branch
        return fake_branch

    def createProject(self, name):
        fake_project = FakeProject(name)
        self.projects[name] = fake_project
        return fake_project

    def searchTasks(self, importance=None, status=None, tags=None):
        # XXX: add tags search support
        search_result = []
        if not importance:
            importance = ALL_IMPORTANCES
        if not status:
            status = OPEN_STATUSES
        for bug in self.bug_tasks:
            if bug.importance in importance and bug.status in status:
                search_result.append(bug)

        return search_result

